# -*- coding: utf-8 -*-
"""
/*
 * This file is part of the pyCVC distribution (https://github.com/polhenarejos/pycvc).
 * Copyright (c) 2022 Pol Henarejos.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
"""

from cryptography.hazmat.primitives.asymmetric import ec
from binascii import unhexlify

class EcCurve(dict):
    P = None
    A = None
    B = None
    G = None
    O = None
    F = None

    def from_name(name):
        for e in EcCurve.__subclasses__():
            if e.__name__.lower() == name.lower():
                return e
        return UnsupportedCurve

    def to_crypto(curve):
        for e in ec.EllipticCurve.__subclasses__():
            if (e.__name__.lower() == curve.__name__.lower()):
                return e()
        return UnsupportedCurve

    def from_P(P):
        for e in EcCurve.__subclasses__():
            if e.P == P:
                return e
        return UnsupportedCurve

class UnsupportedCurve(EcCurve):
    """Not supported"""

class SECP192R1(EcCurve):
    P = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFF"))
    A = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFC"))
    B = bytearray(unhexlify("64210519E59C80E70FA7E9AB72243049FEB8DEECC146B9B1"))
    G = bytearray(unhexlify("04188DA80EB03090F67CBF20EB43A18800F4FF0AFD82FF101207192B95FFC8DA78631011ED6B24CDD573F977A11E794811"))
    O = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFF99DEF836146BC9B1B4D22831"))
    F = bytearray(unhexlify("01"))

class SECP256R1(EcCurve):
    P = bytearray(unhexlify("FFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFF"))
    A = bytearray(unhexlify("FFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFC"))
    B = bytearray(unhexlify("5AC635D8AA3A93E7B3EBBD55769886BC651D06B0CC53B0F63BCE3C3E27D2604B"))
    G = bytearray(unhexlify("046B17D1F2E12C4247F8BCE6E563A440F277037D812DEB33A0F4A13945D898C2964FE342E2FE1A7F9B8EE7EB4A7C0F9E162BCE33576B315ECECBB6406837BF51F5"))
    O = bytearray(unhexlify("FFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551"))
    F = bytearray(unhexlify("01"))

class SECP384R1(EcCurve):
    P = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFF0000000000000000FFFFFFFF"))
    A = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFF0000000000000000FFFFFFFC"))
    B = bytearray(unhexlify("B3312FA7E23EE7E4988E056BE3F82D19181D9C6EFE8141120314088F5013875AC656398D8A2ED19D2A85C8EDD3EC2AEF"))
    G = bytearray(unhexlify("04AA87CA22BE8B05378EB1C71EF320AD746E1D3B628BA79B9859F741E082542A385502F25DBF55296C3A545E3872760AB73617DE4A96262C6F5D9E98BF9292DC29F8F41DBD289A147CE9DA3113B5F0B8C00A60B1CE1D7E819D7A431D7C90EA0E5F"))
    O = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC7634D81F4372DDF581A0DB248B0A77AECEC196ACCC52973"))
    F = bytearray(unhexlify("01"))

class SECP521R1(EcCurve):
    P = bytearray(unhexlify("01FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"))
    A = bytearray(unhexlify("01FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC"))
    B = bytearray(unhexlify("0051953EB9618E1C9A1F929A21A0B68540EEA2DA725B99B315F3B8B489918EF109E156193951EC7E937B1652C0BD3BB1BF073573DF883D2C34F1EF451FD46B503F00"))
    G = bytearray(unhexlify("0400C6858E06B70404E9CD9E3ECB662395B4429C648139053FB521F828AF606B4D3DBAA14B5E77EFE75928FE1DC127A2FFA8DE3348B3C1856A429BF97E7E31C2E5BD66011839296A789A3BC0045C8A5FB42C7D1BD998F54449579B446817AFBD17273E662C97EE72995EF42640C550B9013FAD0761353C7086A272C24088BE94769FD16650"))
    O = bytearray(unhexlify("01FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA51868783BF2F966B7FCC0148F709A5D03BB5C9B8899C47AEBB6FB71E91386409"))
    F = bytearray(unhexlify("01"))


class BrainpoolP192R1(EcCurve):
    P = bytearray(unhexlify("C302F41D932A36CDA7A3463093D18DB78FCE476DE1A86297"))
    A = bytearray(unhexlify("6A91174076B1E0E19C39C031FE8685C1CAE040E5C69A28EF"))
    B = bytearray(unhexlify("469A28EF7C28CCA3DC721D044F4496BCCA7EF4146FBF25C9"))
    G = bytearray(unhexlify("04C0A0647EAAB6A48753B033C56CB0F0900A2F5C4853375FD614B690866ABD5BB88B5F4828C1490002E6773FA2FA299B8F"))
    O = bytearray(unhexlify("C302F41D932A36CDA7A3462F9E9E916B5BE8F1029AC4ACC1"))
    F = bytearray(unhexlify("01"))

class BrainpoolP224R1(EcCurve):
    P = bytearray(unhexlify("D7C134AA264366862A18302575D1D787B09F075797DA89F57EC8C0FF"))
    A = bytearray(unhexlify("68A5E62CA9CE6C1C299803A6C1530B514E182AD8B0042A59CAD29F43"))
    B = bytearray(unhexlify("2580F63CCFE44138870713B1A92369E33E2135D266DBB372386C400B"))
    G = bytearray(unhexlify("040D9029AD2C7E5CF4340823B2A87DC68C9E4CE3174C1E6EFDEE12C07D58AA56F772C0726F24C6B89E4ECDAC24354B9E99CAA3F6D3761402CD"))
    O = bytearray(unhexlify("D7C134AA264366862A18302575D0FB98D116BC4B6DDEBCA3A5A7939F"))
    F = bytearray(unhexlify("01"))

class BrainpoolP256R1(EcCurve):
    P = bytearray(unhexlify("A9FB57DBA1EEA9BC3E660A909D838D726E3BF623D52620282013481D1F6E5377"))
    A = bytearray(unhexlify("7D5A0975FC2C3057EEF67530417AFFE7FB8055C126DC5C6CE94A4B44F330B5D9"))
    B = bytearray(unhexlify("26DC5C6CE94A4B44F330B5D9BBD77CBF958416295CF7E1CE6BCCDC18FF8C07B6"))
    G = bytearray(unhexlify("048BD2AEB9CB7E57CB2C4B482FFC81B7AFB9DE27E1E3BD23C23A4453BD9ACE3262547EF835C3DAC4FD97F8461A14611DC9C27745132DED8E545C1D54C72F046997"))
    O = bytearray(unhexlify("A9FB57DBA1EEA9BC3E660A909D838D718C397AA3B561A6F7901E0E82974856A7"))
    F = bytearray(unhexlify("01"))

class BrainpoolP320R1(EcCurve):
    P = bytearray(unhexlify("D35E472036BC4FB7E13C785ED201E065F98FCFA6F6F40DEF4F92B9EC7893EC28FCD412B1F1B32E27"))
    A = bytearray(unhexlify("3EE30B568FBAB0F883CCEBD46D3F3BB8A2A73513F5EB79DA66190EB085FFA9F492F375A97D860EB4"))
    B = bytearray(unhexlify("520883949DFDBC42D3AD198640688A6FE13F41349554B49ACC31DCCD884539816F5EB4AC8FB1F1A6"))
    G = bytearray(unhexlify("0443BD7E9AFB53D8B85289BCC48EE5BFE6F20137D10A087EB6E7871E2A10A599C710AF8D0D39E2061114FDD05545EC1CC8AB4093247F77275E0743FFED117182EAA9C77877AAAC6AC7D35245D1692E8EE1"))
    O = bytearray(unhexlify("D35E472036BC4FB7E13C785ED201E065F98FCFA5B68F12A32D482EC7EE8658E98691555B44C59311"))
    F = bytearray(unhexlify("01"))

class BrainpoolP384R1(EcCurve):
    P = bytearray(unhexlify("8CB91E82A3386D280F5D6F7E50E641DF152F7109ED5456B412B1DA197FB71123ACD3A729901D1A71874700133107EC53"))
    A = bytearray(unhexlify("7BC382C63D8C150C3C72080ACE05AFA0C2BEA28E4FB22787139165EFBA91F90F8AA5814A503AD4EB04A8C7DD22CE2826"))
    B = bytearray(unhexlify("04A8C7DD22CE28268B39B55416F0447C2FB77DE107DCD2A62E880EA53EEB62D57CB4390295DBC9943AB78696FA504C11"))
    G = bytearray(unhexlify("041D1C64F068CF45FFA2A63A81B7C13F6B8847A3E77EF14FE3DB7FCAFE0CBD10E8E826E03436D646AAEF87B2E247D4AF1E8ABE1D7520F9C2A45CB1EB8E95CFD55262B70B29FEEC5864E19C054FF99129280E4646217791811142820341263C5315"))
    O = bytearray(unhexlify("8CB91E82A3386D280F5D6F7E50E641DF152F7109ED5456B31F166E6CAC0425A7CF3AB6AF6B7FC3103B883202E9046565"))
    F = bytearray(unhexlify("01"))

class BrainpoolP512R1(EcCurve):
    P = bytearray(unhexlify("AADD9DB8DBE9C48B3FD4E6AE33C9FC07CB308DB3B3C9D20ED6639CCA703308717D4D9B009BC66842AECDA12AE6A380E62881FF2F2D82C68528AA6056583A48F3"))
    A = bytearray(unhexlify("7830A3318B603B89E2327145AC234CC594CBDD8D3DF91610A83441CAEA9863BC2DED5D5AA8253AA10A2EF1C98B9AC8B57F1117A72BF2C7B9E7C1AC4D77FC94CA"))
    B = bytearray(unhexlify("3DF91610A83441CAEA9863BC2DED5D5AA8253AA10A2EF1C98B9AC8B57F1117A72BF2C7B9E7C1AC4D77FC94CADC083E67984050B75EBAE5DD2809BD638016F723"))
    G = bytearray(unhexlify("0481AEE4BDD82ED9645A21322E9C4C6A9385ED9F70B5D916C1B43B62EEF4D0098EFF3B1F78E2D0D48D50D1687B93B97D5F7C6D5047406A5E688B352209BCB9F8227DDE385D566332ECC0EABFA9CF7822FDF209F70024A57B1AA000C55B881F8111B2DCDE494A5F485E5BCA4BD88A2763AED1CA2B2FA8F0540678CD1E0F3AD80892"))
    O = bytearray(unhexlify("AADD9DB8DBE9C48B3FD4E6AE33C9FC07CB308DB3B3C9D20ED6639CCA70330870553E5C414CA92619418661197FAC10471DB1D381085DDADDB58796829CA90069"))
    F = bytearray(unhexlify("01"))

class SECP192K1(EcCurve):
    P = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFEE37"))
    A = bytearray(unhexlify("000000000000000000000000000000000000000000000000"))
    B = bytearray(unhexlify("000000000000000000000000000000000000000000000003"))
    G = bytearray(unhexlify("04DB4FF10EC057E9AE26B07D0280B7F4341DA5D1B1EAE06C7D9B2F2F6D9C5628A7844163D015BE86344082AA88D95E2F9D"))
    O = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFE26F2FC170F69466A74DEFD8D"))
    F = bytearray(unhexlify("01"))

class SECP256K1(EcCurve):
    P = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFC2F"))
    A = bytearray(unhexlify("0000000000000000000000000000000000000000000000000000000000000000"))
    B = bytearray(unhexlify("0000000000000000000000000000000000000000000000000000000000000007"))
    G = bytearray(unhexlify("0479BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798483ADA7726A3C4655DA4FBFC0E1108A8FD17B448A68554199C47D08FFB10D4B8"))
    O = bytearray(unhexlify("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364141"))
    F = bytearray(unhexlify("01"))

class Curve25519(EcCurve):
    P = bytearray(unhexlify("7fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffed"))
    A = bytearray(unhexlify("7fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffec"))
    B = bytearray(unhexlify("01"))
    G = bytearray(unhexlify("09"))
    O = bytearray(unhexlify("1000000000000000000000000000000014def9dea2f79cd65812631a5cf5d3ed"))
    F = bytearray(unhexlify("08"))

class Curve448(EcCurve):
    P = bytearray(unhexlify("fffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffffffffffffffffffffffffffffffffffffffffffffffffffff"))
    A = bytearray(unhexlify("0262a6"))
    B = bytearray(unhexlify("01"))
    G = bytearray(unhexlify("05"))
    O = bytearray(unhexlify("3fffffffffffffffffffffffffffffffffffffffffffffffffffffff7cca23e9c44edb49aed63690216cc2728dc58f552378c292ab5844f3"))
    F = bytearray(unhexlify("04"))
